from src.dial import DIscoveryAndLaunch
from unittest import TestCase


class testDIscoveryAndLaunch(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass


